/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */
/*
 * Copyright 2013 Saso Kiselkov.  All rights reserved.
 * Use is subject to license terms.
 */
#include <sys/zfs_context.h>
#include <sys/zio.h>
#include <sys/edonr.h>

/*
 * zio_checksum interface for the Edon-R hash function
 */
void
zio_checksum_EdonR512_256(const void *buf, uint64_t size, zio_cksum_t *zcp)
{
	uint8_t hash[EdonR512_DIGEST_SIZE];

	EdonRHash(512, buf, size * 8, hash);
	bcopy(&hash[000], &zcp->zc_word[0], sizeof (zcp->zc_word[0]));
	bcopy(&hash[010], &zcp->zc_word[1], sizeof (zcp->zc_word[0]));
	bcopy(&hash[020], &zcp->zc_word[2], sizeof (zcp->zc_word[0]));
	bcopy(&hash[030], &zcp->zc_word[3], sizeof (zcp->zc_word[0]));
}

void
zio_checksum_EdonR512_256_byteswap(const void *buf, uint64_t size,
    zio_cksum_t *zcp)
{
	zio_checksum_EdonR512_256(buf, size, zcp);
	zcp->zc_word[0] = BSWAP_64(zcp->zc_word[0]);
	zcp->zc_word[1] = BSWAP_64(zcp->zc_word[1]);
	zcp->zc_word[2] = BSWAP_64(zcp->zc_word[2]);
	zcp->zc_word[3] = BSWAP_64(zcp->zc_word[3]);
}
